package jp.ne.goo.mail.orangewind;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.aiwolf.client.base.player.AbstractBodyguard;
import org.aiwolf.client.lib.TemplateTalkFactory;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

import jp.ne.goo.mail.orangewind.Util.GameStatus;

public class MKBodyguard extends AbstractBodyguard {
	//狩人
	protected final boolean isRethrowException = false; //例外を吐くか？

	GameStatus gameStatus = new GameStatus(this);

	//COした？
	boolean isCO = false;

	//会話チェック
	boolean isSaidGuardData = false;
	int estimateCount = 0; //感想を述べた人の数

	@Override
	public void initialize(GameInfo gameInfo, GameSetting gameSetting){
		super.initialize(gameInfo, gameSetting);

		gameStatus = new GameStatus(this);
		isCO = false;
	}

	@Override
	public void dayStart() {
		// GameStatusを更新

		initTalkFlag();
	}

	@Override
	public void finish() {
		// 後始末

	}

	@Override
	public void update(GameInfo gameInfo){
		try{
			super.update(gameInfo);

			gameStatus.addGameStatus(gameInfo);
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }
		}
	}

	@Override
	public Agent guard() {
		try{
			// 占いCOした人の中から守る
			// 占いがいないか、3-1の場合は霊媒
			// 霊媒もいなければ暫定白を
			List<Agent> seers = gameStatus.getOtherSeer();
			List<Agent> mediums = gameStatus.getOtherMedium();
			if(mediums.size() == 1 && seers.size() > 2){
				return randomSelect(mediums);
			}else if (seers.size() > 0){
				return randomSelect(seers);
			}else if (mediums.size() > 0){
				return randomSelect(mediums);
			}else{
				if(gameStatus.divinedHumanAgents().size() > 0){
					return randomSelect(gameStatus.divinedHumanAgents());
				}else{
					//暫定白もいなければ、もう誰でもいい

					//護衛対象リスト
					List<Agent> guardCandidates = new ArrayList<Agent>();

					//生きてればリストに
					guardCandidates.addAll(getLatestDayGameInfo().getAliveAgentList());

					//自分を外す
					guardCandidates.remove(getMe());

					return randomSelect(guardCandidates);
				}
			}
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }else{
	        	List<Agent> tempList = getLatestDayGameInfo().getAliveAgentList();
	        	tempList.remove(getMe());
	        	return randomSelect(tempList); //自分を返す可能性はあるが仕方ない
	        }
		}
	}

	@Override
	public String talk() {
		try{
			// 基本的にCOしない
			// 吊られそうな流れになったら（別々のAgentが自分にvoteする発言を過半数したら）狩人CO

			if(isCO){
				if(!isSaidGuardData){
					return TemplateTalkFactory.guarded(getLatestDayGameInfo().getGuardedAgent());
				}else{
					return TemplateTalkFactory.over();
				}
			}else{
				//占い師から自分に黒出しされた場合
				if(gameStatus.getEnemySeerList().size() > 0){
					//狩人CO
					isCO = true;
					return TemplateTalkFactory.comingout(getMe(), Role.BODYGUARD);
				}

				//適当な白出し
				if(estimateCount < 1){
					estimateCount++;
					return TemplateTalkFactory.estimate(randomSelect(villagerList()), Role.VILLAGER);
				}else{
					return TemplateTalkFactory.over();
				}
			}
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }else{
	        	return TemplateTalkFactory.over();
	        }
		}
	}

	@Override
	public Agent vote() {
		try{
			Agent votedAgent = gameStatus.mostVoted();
			if (votedAgent != null && votedAgent != this.getMe()){
				return votedAgent;
			}else{
				//上手く取れなかったり、自分が疑われてたりしたらランダムで……。

				//投票対象リスト
				List<Agent> voteCandidates = new ArrayList<Agent>();

				//生きてればリストに
				voteCandidates.addAll(getLatestDayGameInfo().getAliveAgentList());

				//自分を外す
				voteCandidates.remove(getMe());

				return randomSelect(voteCandidates);
			}
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }else{
	        	return getMe();
	        }
		}
	}

	private void initTalkFlag(){
		isSaidGuardData = false;
		estimateCount = 0;
	}

	//リスト渡すとランダムでAgent選択する
	private Agent randomSelect(List<Agent> agentList){
		if(agentList.size() > 0){
			int num = new Random().nextInt(agentList.size());
			return agentList.get(num);
		}else{
			return getMe(); //ダメだったらとりあえず自分
		}
	}

	/**
	 * 自分とCOした人間を除いた生存者リスト
	 * @param agentList
	 * @return
	 */
	private List<Agent> villagerList(){
		List<Agent> aliveVillagerList = new ArrayList<Agent>();
		aliveVillagerList.addAll(getLatestDayGameInfo().getAliveAgentList());
		aliveVillagerList.remove(getMe());

		//ここでCOした人を除いたリストをどうにか取得する
		//いなかった場合は……自分でいいか？

		return aliveVillagerList;
	}
}
