package jp.ne.goo.mail.orangewind;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.aiwolf.client.base.player.AbstractMedium;
import org.aiwolf.client.lib.TemplateTalkFactory;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Judge;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

import jp.ne.goo.mail.orangewind.Util.GameStatus;

/**
 * 霊媒
 * @author M.K.
 *
 */
public class MKMedium extends AbstractMedium {
	protected final boolean isRethrowException = false; //例外を吐くか？

	//霊媒した人のリストはthis.getMyJudgeList
	GameStatus gameStatus = new GameStatus(this);

	//COした？
	boolean isCO = false;

	//会話チェック
	boolean isSaidMediumData = false;
	int estimateCount = 0; //感想を述べた人の数

	@Override
	public void initialize(GameInfo gameInfo, GameSetting gameSetting){
		super.initialize(gameInfo, gameSetting);

		gameStatus = new GameStatus(this);
		isCO = false;
	}

	@Override
	public void finish() {
		// ゲーム終了時の処理
	}

	@Override
	public void dayStart() {
		try{
			// 一日の始まり
			// 霊媒結果の取得
			super.dayStart();

			//生きてる自分以外の人のリスト
			List<Agent> aliveAgentList = new ArrayList<Agent>();
			aliveAgentList.addAll(getLatestDayGameInfo().getAliveAgentList());
			aliveAgentList.remove(getMe());

			//会話状態のリセット
			initTalkFlag();

			// GameStatusの更新
			gameStatus.addGameStatus(getLatestDayGameInfo());
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }
		}
	}

	@Override
	public void update(GameInfo gameInfo){
		try{
			super.update(gameInfo);

			gameStatus.addGameStatus(gameInfo);
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }
		}
	}

	@Override
	public String talk() {
		try{
			if(this.getDay() <= 1){
				//初日、1日目は潜伏
				//ただし、吊られそうな場合はCO
				Agent votedAgent = gameStatus.mostVoted();
				if(votedAgent == getMe() && !isCO){
					return TemplateTalkFactory.comingout(getMe(), getMyRole()); //私は霊媒です
				}
				return TemplateTalkFactory.over();
			}else if (this.getDay() == 2 && !isCO){
				isCO = true;
				return TemplateTalkFactory.comingout(getMe(), getMyRole()); //私は霊媒です
			}else if (!isSaidMediumData){
				Judge lastDayJudge = this.getLatestDayGameInfo().getMediumResult(); //昨日の結果を取得
				isSaidMediumData = true;
				return TemplateTalkFactory.inquested(lastDayJudge.getTarget(), lastDayJudge.getResult());
			}else{
				//適当な白出し
				if (estimateCount < 2){
					estimateCount++;
					Agent randomVillager = randomSelect(villagerList());
					if(randomVillager != null && randomVillager != getMe()){
						return TemplateTalkFactory.estimate(randomVillager, Role.VILLAGER);
					}else{
						return TemplateTalkFactory.skip();
					}
				}else{
					return TemplateTalkFactory.over();
				}
			}
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }else{
	        	return TemplateTalkFactory.over();
	        }
		}

//8/4 23:59版
/*		//以下をランダムで
		//霊媒潜伏はリスク高いのでしなくていいと思うが、初日潜伏はしてよい
		List<String> talkList = new ArrayList<String>();

		//2日目のみ？
		talkList.add(TemplateTalkFactory.comingout(getMe(), getMyRole())); //私は霊媒です

		//霊媒結果の報告。正直これだけやってればいいような気がする
		talkList.add(TemplateTalkFactory.inquested(getMe(), Species.HUMAN)); //これちゃんと作らないとな……

		talkList.add(TemplateTalkFactory.skip()); //様子見

		talkList.add(TemplateTalkFactory.over()); //おしまい

		return randomTalkSelect(talkList);*/
	}

	@Override
	public Agent vote() {
		try{
			//対抗霊媒がいればそちらに
			List<Agent> otherMedium = gameStatus.getOtherMedium();
			if (otherMedium.size() > 0){
				return randomSelect(otherMedium);
			}else if(gameStatus.divinedWerewolfAgents().size() > 0){
				//黒確と目されてるのがいれば
				return randomSelect(gameStatus.divinedWerewolfAgents());
			}else{
				//村人と同じ処理
				Agent votedAgent = gameStatus.mostVoted();
				if (votedAgent != null && votedAgent != this.getMe()){
					return votedAgent;
				}else{
					//上手く取れなかったり、自分が疑われてたりしたらランダムで

					//投票対象リスト
					List<Agent> voteCandidates = new ArrayList<Agent>();

					//生きてればリストに
					voteCandidates.addAll(getLatestDayGameInfo().getAliveAgentList());

					//自分を外す
					voteCandidates.remove(getMe());

					return randomSelect(voteCandidates);
				}
			}
		}catch(Exception ex){
	        if(isRethrowException){
	            throw ex;
	        }else{
	        	return getMe();
	        }
		}
	}

	private void initTalkFlag(){
		isSaidMediumData = false;
		estimateCount = 0;
	}

	//リスト渡すとランダムでAgent選択する
	private Agent randomSelect(List<Agent> agentList){
		int num = new Random().nextInt(agentList.size());
		return agentList.get(num);
	}

	/**
	 * 自分とCOした人間を除いた生存者リスト
	 * @param agentList
	 * @return
	 */
	private List<Agent> villagerList(){
		List<Agent> aliveVillagerList = new ArrayList<Agent>();
		aliveVillagerList.addAll(getLatestDayGameInfo().getAliveAgentList());
		aliveVillagerList.remove(getMe());

		//COした人を除いたリストを取得する
		aliveVillagerList.remove(gameStatus.getOtherMedium());
		aliveVillagerList.remove(gameStatus.getOtherSeer());

		//なお、いなかった場合は自分を返す
		if(aliveVillagerList.size() == 0){
			aliveVillagerList.add(getMe());
		}

		return aliveVillagerList;
	}

}
