package jp.ne.goo.mail.orangewind.Util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.aiwolf.client.base.player.AbstractRole;
import org.aiwolf.client.lib.Utterance;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.data.Talk;
import org.aiwolf.common.net.GameInfo;

public class GameStatus {
	//いろいろ持っておく

	//親クラス
	AbstractRole myself;

	//占いCOした人のリスト
	List<Agent> seerCOList;

	//霊媒COした人のリスト
	List<Agent> mediumCOList;

	//村人以外のCOリスト
	private List<Agent> otherCOList;

	//人狼COリスト
	List<Agent> wolfCOList;

	//襲撃死した人のリスト（白確）
	List<Agent> killedList;

	//自分に黒出しした占い師のリスト
	private List<Agent> enemySeerList;

	//占い結果リスト
	Map<Agent, Map<Species, List<Agent>>> seerResults;

	//ターンごとの投票先リスト
	//key=day item=投票先Map(key=Agent item=投票先Agent)
	Map<Integer, Map<Agent, Agent>> voteDataList;

	//ターンごとの読み込んだtalk番号リスト
	Map<Integer, Integer> readTalkList;



	public GameStatus(AbstractRole me){
		this.seerCOList = new ArrayList<Agent>();
		this.mediumCOList = new ArrayList<Agent>();
		this.setOtherCOList(new ArrayList<Agent>());
		this.wolfCOList = new ArrayList<Agent>();

		this.killedList = new ArrayList<Agent>();
		this.setEnemySeerList(new ArrayList<Agent>());

		this.voteDataList = new HashMap<Integer, Map<Agent, Agent>>();
		this.readTalkList = new HashMap<Integer, Integer>();
		this.seerResults = new HashMap<Agent, Map<Species, List<Agent>>>();

		this.myself = me;
	}

	public void addGameStatus(GameInfo gameInfo){
		killedList.add(gameInfo.getAttackedAgent()); //襲撃リストに追加

		//まだ今日のMapがなければ作る
		//投票先
		if (!voteDataList.containsKey(gameInfo.getDay())){
			voteDataList.put(gameInfo.getDay(), new HashMap<Agent, Agent>());
		}
		//ここまで読んだ
		if (!readTalkList.containsKey(gameInfo.getDay())){
			readTalkList.put(gameInfo.getDay(), -1); //talk番号は0はじまりなので
		}

		//ひとつずつ読みながら、各種COリストと投票先リストに入れていく
		for(Talk talk:gameInfo.getTalkList()){
			if (talk.getIdx() < readTalkList.get(talk.getDay())){
				continue;
			}

			if (readTalkList.get(talk.getDay()) < talk.getIdx()){
				//条件は念のため
				readTalkList.put(talk.getDay(), talk.getIdx()); //このtalk読んだ
			}

			Utterance utterance = new Utterance(talk.getContent());

			Map<Agent, Agent> todayVoteList = voteDataList.get(talk.getDay()); //今日の投票先リスト

			switch(utterance.getTopic()){
			case COMINGOUT:
				Role coRole = utterance.getRole();
				if(coRole == Role.SEER){
					seerCOList.add(talk.getAgent()); //占いCO
				}else if (coRole == Role.MEDIUM){
					mediumCOList.add(talk.getAgent()); //霊媒CO
				}else if (coRole == Role.BODYGUARD || coRole == Role.POSSESSED){
					getOtherCOList().add(talk.getAgent()); //この辺はまず起きないと思うのですが
				}else if (coRole == Role.WEREWOLF){
					wolfCOList.add(talk.getAgent()); //この辺はまず起きないと思うのですが
				}
				//村人COにはあんまり意味がないと思うのでまあいい
				break;
			case VOTE:
				todayVoteList.put(talk.getAgent(), utterance.getTarget());
				break;
			case ESTIMATE:
				//ひとまず黒出しのみチェック
				//VOTEと同じ扱いで
				//すでにVOTE発言していたら、そっちを優先
				if (utterance.getRole() == Role.WEREWOLF){
					if (!todayVoteList.containsKey(talk.getAgent())){
						todayVoteList.put(talk.getAgent(), utterance.getTarget()); //投票者、投票先
					}
				}
				break;
			case DIVINED:
				if (utterance.getResult() == Species.WEREWOLF){
					if(utterance.getTarget() == myself.getMe()){
						getEnemySeerList().add(talk.getAgent()); //自分に黒出しした占い師
					}
					//それはそれとして
					//占われリスト
					if (!seerResults.containsKey(utterance.getTarget())){
						Map<Species, List<Agent>> newMap = new HashMap<Species, List<Agent>>();
						newMap.put(Species.HUMAN, new ArrayList<Agent>());
						newMap.put(Species.WEREWOLF, new ArrayList<Agent>());
						seerResults.put(utterance.getTarget(), newMap);
					}
					Map<Species, List<Agent>> tempMap = seerResults.get(utterance.getTarget());
					List<Agent> tempList = tempMap.get(utterance.getResult());
					tempList.add(talk.getAgent());
				}
				break;
			case INQUESTED:
				//ライン処理は……気にしない方向で
			case GUARDED:
			case ATTACK:
				break;
			case AGREE:
				//使う人は少ないと思うので、とりあえず実装後回し
			case DISAGREE:
				//同上。人狼知能のルールでは説得ができないのでほぼ無意味かな？と
			case OVER:
			case SKIP:
				break;
			default:
				break;
			}
		}
		return;
	}

	//以下Util

	//生存中の(対抗)占いの取得
	public List<Agent> getOtherSeer(){
		List<Agent> aliveSeerCOList = aliveList(seerCOList);

		aliveSeerCOList.remove(myself.getMe());

		return aliveSeerCOList;
	}

	//生存中の(対抗)霊媒の取得
	public List<Agent> getOtherMedium(){
		List<Agent> aliveMediumCOList = aliveList(mediumCOList);

		aliveMediumCOList.remove(myself.getMe());

		return aliveMediumCOList;
	}

	//リストの中から生存者だけ返す
	private List<Agent> aliveList(List<Agent> agentList){
		List<Agent> aliveList = new ArrayList<Agent>();
		List<Agent> aliveAgents = myself.getLatestDayGameInfo().getAliveAgentList();
		for(Agent agent: agentList){
			if (aliveAgents.contains(agent)){
				aliveList.add(agent);
			}
		}

		return aliveList;
	}

	//一番投票されてる人（吊られそうな人）
	//同点のときは適当に
	//最新だけ取ってくるはずなので、変更した人がいても大丈夫なはず……。
	public Agent mostVoted(String command){
		Map<Agent, Integer> count = new HashMap<Agent, Integer>();
		if (command.equals("")){
			Map<Agent, Agent> todayVoteList = voteDataList.get(myself.getDay());
			for(Agent agent:todayVoteList.values()){
				if(count.containsKey(agent)){
					int num = count.get(agent);
					count.put(agent, num + 1);
				}else{
					count.put(agent, 1);
				}
			}

			int max = 0;
			Agent maxAgent = null;
			for(Map.Entry<Agent, Integer> entry: count.entrySet()){
				if (max < entry.getValue()){
					maxAgent = entry.getKey();
					max = entry.getValue();
				}
			}
			return maxAgent;
		}else{
			//何か他の処理をしたければ
			Map<Agent, Agent> todayVoteList = voteDataList.get(myself.getDay());
			for(Agent agent:todayVoteList.values()){
				if(count.containsKey(agent)){
					int num = count.get(agent);
					count.put(agent, num + 1);
				}else{
					count.put(agent, 1);
				}
			}

			int max = 0;
			List<Agent> maxVotedAgentList = new ArrayList<Agent>();
			for(Map.Entry<Agent, Integer> entry: count.entrySet()){
				if (max < entry.getValue()){
					maxVotedAgentList = new ArrayList<Agent>();
					maxVotedAgentList.add(entry.getKey());
					max = entry.getValue();
				}else if (max == entry.getValue()){
					maxVotedAgentList.add(entry.getKey());
				}
			}
			if (maxVotedAgentList.size() == 0){
				//誰も投票していないなど
				return null;
			}
			Agent maxAgent = null;

			if (command.equals("villager")){
				for (Agent agent: maxVotedAgentList){
					if(!seerCOList.contains(agent) && !mediumCOList.contains(agent)){
						maxAgent = agent;
						break;
					}
				}
				// 該当なしの場合は適当に
				if (maxAgent == null && maxVotedAgentList.size() > 0){
					return maxVotedAgentList.get(0);
				}else{
					return mostVoted(); //一応
				}
			}else{
				return mostVoted();
			}
		}
	}

	public Agent mostVoted(){
		return this.mostVoted("");
	}

	//一番投票されてない人
	public Agent leastVoted(){
		Map<Agent, Integer> count = new HashMap<Agent, Integer>();

		Map<Agent, Agent> todayVoteList = voteDataList.get(myself.getDay());
		for(Agent agent:todayVoteList.values()){
			if(count.containsKey(agent)){
				int num = count.get(agent);
				count.put(agent, num + 1);
			}else{
				count.put(agent, 1);
			}
		}

		int min = 100;
		Agent minAgent = null;
		for(Map.Entry<Agent, Integer> entry: count.entrySet()){
			if (min > entry.getValue()){
				minAgent = entry.getKey();
			}
		}
		return minAgent;
	}

	//一番投票されてない人(指定された中で)
	public Agent leastVoted(List<Agent> agentList){
		if(agentList.size() == 0){
			return null;
		}

		Map<Agent, Integer> count = new HashMap<Agent, Integer>();

		Map<Agent, Agent> todayVoteList = voteDataList.get(myself.getDay());
		for(Agent agent:todayVoteList.values()){
			if(count.containsKey(agent)){
				int num = count.get(agent);
				count.put(agent, num + 1);
			}else{
				count.put(agent, 1);
			}
		}

		int min = 100;
		Agent minAgent = null;
		for(Agent agent:agentList){
			if (!count.containsKey(agent)){
				min = 0;
				minAgent = agent;
			}
			else
			{
				if (min > count.get(agent)){
					minAgent = agent;
					min = count.get(agent);
				}
			}
		}
		return minAgent;
	}

	//占いによれば暫定白の生存者
	public List<Agent> divinedHumanAgents(){
		List<Agent> divined = new ArrayList<Agent>();
		for (Agent agent:myself.getLatestDayGameInfo().getAliveAgentList()){
			if(seerResults.containsKey(agent)){
				Map<Species, List<Agent>> result = seerResults.get(agent);
				if (result.get(Species.HUMAN).size() > 0 && result.get(Species.WEREWOLF).size() == 0){
					//暫定白
					divined.add(agent);
				}
			}
		}
		return divined;
	}

	//占いによれば暫定黒の生存者
	public List<Agent> divinedWerewolfAgents(){
		List<Agent> divined = new ArrayList<Agent>();
		for (Agent agent:myself.getLatestDayGameInfo().getAliveAgentList()){
			if(seerResults.containsKey(agent)){
				Map<Species, List<Agent>> result = seerResults.get(agent);
				if (result.get(Species.HUMAN).size() == 0 && result.get(Species.WEREWOLF).size() > 0){
					//暫定黒
					divined.add(agent);
				}
			}
		}
		return divined;
	}

	//占われてない生存者
	public List<Agent> notDivinedAgents(){
		List<Agent> divined = new ArrayList<Agent>();
		for (Agent agent:myself.getLatestDayGameInfo().getAliveAgentList()){
			if(seerResults.containsKey(agent)){
				Map<Species, List<Agent>> result = seerResults.get(agent);
				if (result.get(Species.HUMAN).size() == 0 && result.get(Species.WEREWOLF).size() == 0){
					//ここには来ないと思うけど、占われてない人
					divined.add(agent);
				}
			}else{
				divined.add(agent);
			}
		}
		return divined;
	}

	//パンダの生存者
	public List<Agent> divinedPandaAgents(){
		List<Agent> divined = new ArrayList<Agent>();
		for (Agent agent:myself.getLatestDayGameInfo().getAliveAgentList()){
			if(seerResults.containsKey(agent)){
				Map<Species, List<Agent>> result = seerResults.get(agent);
				if (result.get(Species.HUMAN).size() > 0 && result.get(Species.WEREWOLF).size() > 0){
					//パンダ
					divined.add(agent);
				}
			}
		}
		return divined;
	}



	//以下Getter・Setter
	public List<Agent> getEnemySeerList() {
		return enemySeerList;
	}

	public void setEnemySeerList(List<Agent> enemySeerList) {
		this.enemySeerList = enemySeerList;
	}

	public List<Agent> getOtherCOList() {
		return otherCOList;
	}

	public void setOtherCOList(List<Agent> otherCOList) {
		this.otherCOList = otherCOList;
	}
}
